﻿using System;
using System.Linq;
using System.Text;

namespace SharpChat {
    public class ChatChannel {
        public string Name { get; set; }
        public string Password { get; set; } = string.Empty;
        public bool IsTemporary { get; set; } = false;
        public int Rank { get; set; } = 0;
        public ChatUser Owner { get; set; } = null;

        public bool HasPassword
            => !string.IsNullOrWhiteSpace(Password);

        public ChatChannel() { }

        public ChatChannel(string name) {
            Name = name;
        }

        public string Pack() {
            StringBuilder sb = new();

            sb.Append(Name);
            sb.Append('\t');
            sb.Append(string.IsNullOrEmpty(Password) ? '0' : '1');
            sb.Append('\t');
            sb.Append(IsTemporary ? '1' : '0');

            return sb.ToString();
        }

        public bool NameEquals(string name) {
            return string.Equals(name, Name, StringComparison.InvariantCultureIgnoreCase);
        }

        public override int GetHashCode() {
            return Name.GetHashCode();
        }

        public static bool CheckName(string name) {
            return !string.IsNullOrWhiteSpace(name) && name.All(CheckNameChar);
        }

        public static bool CheckNameChar(char c) {
            return char.IsLetter(c) || char.IsNumber(c) || c == '-' || c == '_';
        }
    }
}
