﻿using SharpChat.Packet;
using System.Linq;

namespace SharpChat.Commands {
    public class DeleteChannelCommand : IChatCommand {
        public bool IsMatch(ChatCommandContext ctx) {
            return ctx.NameEquals("delchan") || (
                ctx.NameEquals("delete")
                && ctx.Args.FirstOrDefault()?.All(char.IsDigit) == false
            );
        }

        public void Dispatch(ChatCommandContext ctx) {
            if(!ctx.Args.Any() || string.IsNullOrWhiteSpace(ctx.Args.FirstOrDefault())) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.COMMAND_FORMAT_ERROR));
                return;
            }

            string delChanName = string.Join('_', ctx.Args);
            ChatChannel delChan = ctx.Chat.Channels.FirstOrDefault(c => c.NameEquals(delChanName));

            if(delChan == null) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.CHANNEL_NOT_FOUND, true, delChanName));
                return;
            }

            if(!ctx.User.Can(ChatUserPermissions.DeleteChannel) && delChan.Owner != ctx.User) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.CHANNEL_DELETE_FAILED, true, delChan.Name));
                return;
            }

            ctx.Chat.RemoveChannel(delChan);
            ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.CHANNEL_DELETED, false, delChan.Name));
        }
    }
}
