﻿using SharpChat.Misuzu;
using SharpChat.Packet;
using System;
using System.Linq;
using System.Net;
using System.Threading.Tasks;

namespace SharpChat.Commands {
    public class PardonAddressCommand : IChatCommand {
        private readonly MisuzuClient Misuzu;

        public PardonAddressCommand(MisuzuClient msz) {
            Misuzu = msz ?? throw new ArgumentNullException(nameof(msz));
        }

        public bool IsMatch(ChatCommandContext ctx) {
            return ctx.NameEquals("pardonip")
                || ctx.NameEquals("unbanip");
        }

        public void Dispatch(ChatCommandContext ctx) {
            if(!ctx.User.Can(ChatUserPermissions.BanUser | ChatUserPermissions.KickUser)) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.COMMAND_NOT_ALLOWED, true, $"/{ctx.Name}"));
                return;
            }

            string unbanAddrTarget = ctx.Args.FirstOrDefault();
            if(string.IsNullOrWhiteSpace(unbanAddrTarget) || !IPAddress.TryParse(unbanAddrTarget, out IPAddress unbanAddr)) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.COMMAND_FORMAT_ERROR));
                return;
            }

            unbanAddrTarget = unbanAddr.ToString();

            Task.Run(async () => {
                MisuzuBanInfo banInfo = await Misuzu.CheckBanAsync(ipAddr: unbanAddrTarget);

                if(!banInfo.IsBanned || banInfo.HasExpired) {
                    ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.USER_NOT_BANNED, true, unbanAddrTarget));
                    return;
                }

                bool wasBanned = await Misuzu.RevokeBanAsync(banInfo, MisuzuClient.BanRevokeKind.RemoteAddress);
                if(wasBanned)
                    ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.USER_UNBANNED, false, unbanAddrTarget));
                else
                    ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.USER_NOT_BANNED, true, unbanAddrTarget));
            }).Wait();
        }
    }
}
