﻿using SharpChat.Misuzu;
using SharpChat.Packet;
using System;
using System.Linq;
using System.Threading.Tasks;

namespace SharpChat.Commands {
    public class PardonUserCommand : IChatCommand {
        private readonly MisuzuClient Misuzu;

        public PardonUserCommand(MisuzuClient msz) {
            Misuzu = msz ?? throw new ArgumentNullException(nameof(msz));
        }

        public bool IsMatch(ChatCommandContext ctx) {
            return ctx.NameEquals("pardon")
                || ctx.NameEquals("unban");
        }

        public void Dispatch(ChatCommandContext ctx) {
            if(!ctx.User.Can(ChatUserPermissions.BanUser | ChatUserPermissions.KickUser)) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.COMMAND_NOT_ALLOWED, true, $"/{ctx.Name}"));
                return;
            }

            bool unbanUserTargetIsName = true;
            string unbanUserTarget = ctx.Args.FirstOrDefault();
            if(string.IsNullOrWhiteSpace(unbanUserTarget)) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.COMMAND_FORMAT_ERROR));
                return;
            }

            ChatUser unbanUser = ctx.Chat.Users.FirstOrDefault(u => u.NameEquals(unbanUserTarget));
            if(unbanUser == null && long.TryParse(unbanUserTarget, out long unbanUserId)) {
                unbanUserTargetIsName = false;
                unbanUser = ctx.Chat.Users.FirstOrDefault(u => u.UserId == unbanUserId);
            }

            if(unbanUser != null)
                unbanUserTarget = unbanUser.UserId.ToString();

            Task.Run(async () => {
                MisuzuBanInfo banInfo = await Misuzu.CheckBanAsync(unbanUserTarget, userIdIsName: unbanUserTargetIsName);

                if(!banInfo.IsBanned || banInfo.HasExpired) {
                    ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.USER_NOT_BANNED, true, unbanUserTarget));
                    return;
                }

                bool wasBanned = await Misuzu.RevokeBanAsync(banInfo, MisuzuClient.BanRevokeKind.UserId);
                if(wasBanned)
                    ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.USER_UNBANNED, false, unbanUserTarget));
                else
                    ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.USER_NOT_BANNED, true, unbanUserTarget));
            }).Wait();
        }
    }
}
