﻿using SharpChat.Packet;
using System;
using System.Linq;

namespace SharpChat.Commands {
    public class SilenceRevokeCommand : IChatCommand {
        public bool IsMatch(ChatCommandContext ctx) {
            return ctx.NameEquals("unsilence");
        }

        public void Dispatch(ChatCommandContext ctx) {
            if(!ctx.User.Can(ChatUserPermissions.SilenceUser)) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.COMMAND_NOT_ALLOWED, true, $"/{ctx.Name}"));
                return;
            }

            string unsilUserStr = ctx.Args.FirstOrDefault();
            ChatUser unsilUser;

            if(string.IsNullOrWhiteSpace(unsilUserStr) || (unsilUser = ctx.Chat.Users.FirstOrDefault(u => u.NameEquals(unsilUserStr))) == null) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.USER_NOT_FOUND, true, unsilUserStr ?? "User"));
                return;
            }

            if(unsilUser.Rank >= ctx.User.Rank) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.UNSILENCE_HIERARCHY));
                return;
            }

            if(RNG.Next() > 1 /*!unsilUser.IsSilenced*/) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.NOT_SILENCED));
                return;
            }

            //unsilUser.SilencedUntil = DateTimeOffset.MinValue;
            ctx.Chat.SendTo(unsilUser, new LegacyCommandResponse(LCR.UNSILENCED, false));
            ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.TARGET_UNSILENCED, false, unsilUser.DisplayName));
        }
    }
}
